function fullcalib=xyzuvcal_both(xyzuvname,finname)
% xyzuvcal_both - calibrate both int/ext camera params from xyzuv
%
%    function fullcalib=xyzuvcal_both(xyzuvname,finname)
%
%    Derived from xyzuv2fin, but changed so that input and 
%    output files are explicitly named, rather than implicit
% 
%    xyzuvname - the name of the input file
%    finname   - the name of the output file
%
%    It returns a struct that contains mean,std,max error (and
%    corresponding index)
%
%    JED 11/00

if nargin<2
  disp('Usage: xyzuvcal_both <xyzuvfilename> <outputfilename>');
  fullcalib=[];
  return;
end

cameratype='generic';
inname=xyzuvname;
xyzuvdata=loadascii(inname);
fprintf('\n%s loaded.\n',inname);
[par,pos,iter,res,er,C]=cacal(cameratype,xyzuvdata);
sys=configc(cameratype);
a=invmodel(cameratype,par);
fullpar=[sys(1:4);par;a;pos];

fname=finname;
%save(fname,'fullpar','-ascii')
saveascii(fname,fullpar)
fprintf(1,'\nIntrinsic and extrinsic paramters save in %s.\n',fname);

% report errors we need:
n=size(er,1)/2;
er2=reshape(er,n,2);
sq_error = er2(:,1).*er2(:,1) + er2(:,2).*er2(:,2);
abs_error = sqrt(sq_error);
mean_error = mean(abs_error);
std_error = std(abs_error);
fprintf(1,'\nMean absolute error: %f [pix]\n',mean_error);
fprintf(1,'STD of absolute error: %f [pix]\n',std_error);
[m, i]=max(abs_error);
fprintf(1,'Max absolute error: %f [pix], occuring at index %f.\n',m,i);

fullcalib.fullpar = fullpar;
fullcalib.mean_error=mean_error;
fullcalib.std_error = std_error;
fullcalib.max_error = m;
fullcalib.max_idx = i;
