function extrcalib=xyzuvcal_extonly(xyzuvname,intrname,finname)
% XYZUVCAL_EXTONLY - find the ext cam given xyzuv and int params
%
%    function extrcalib=xyzuvcal_extonly(xyzuvname,intrname,finname)
%
%    Derived from xyzuv_intr2fin, but modified to explicitly name
%    all the files involved.
%
%    xyzuvname - the input xyzuv data file
%    intrname  - the name of a file with internal camera parameters
%    finname   - the output fin file
%
%    estimates the extrinsic parameters from a .xyzuv file and a .fin file.
%    The .fin file providess the intrinsic paramters. and
%    the .xyzuv file provides 3D-2D correspondances
%    The intrinsic and extrinsic parameters are written in a .fin
%    in the directory specified.
%
%    JED 11/00

if nargin<3
  disp('Usage: xyzuvcal_extonly <in.xyzuv> <internal.fin> <output.fin>');
  fullcalib=[];
  return;
end

fname=xyzuvname;
xyzuvdata=loadascii(fname);
fprintf('\n%s loaded.\n',fname);


intrinsic=loadascii(intrname);
fprintf('\n%s loaded.\n',fname);

cameratype='generic';
intrinsic=intrinsic(1:20);% cut off extrinsics
par=intrinsic(5:12);
[pos,iter,res,er,C]=extcal(cameratype,xyzuvdata,par);

fullpar=[intrinsic;pos];

fname=finname;
%feval('save',fname,'fullpar','-ascii')
saveascii(fname,fullpar);
fprintf(1,'\nIntrinsic and extrinsic paramters saved in %s.\n',fname);

% report errors we need:
n=size(er,1)/2;
er2=reshape(er,n,2);
sq_error = er2(:,1).*er2(:,1) + er2(:,2).*er2(:,2);
abs_error = sqrt(sq_error);
mean_error = mean(abs_error);
std_error = std(abs_error);
fprintf(1,'\nMean absolute error: %f [pix]\n',mean_error);
fprintf(1,'STD of absolute error: %f [pix]\n',std_error);
[m, i]=max(abs_error);
fprintf(1,'Max absolute error: %f [pix], occuring at index %f.\n',m,i);

extrcalib.fullpar = fullpar;
extrcalib.mean_error=mean_error;
extrcalib.std_error = std_error;
extrcalib.max_error = m;
extrcalib.max_idx = i;

