
#include <iostream.h>
#include <stdio.h>
#include <math.h>

#include "Image.h"
#include "hough.h"

#define SHOW(x) cout << #x " = " << x << endl

#define MIN(x, y) (((x)<(y)) ? (x) : (y))
#define MAX(x, y) (((x)>(y)) ? (x) : (y))

#define DEBUG 0

void
showMsg(char *s)
{
  cerr << s << endl;
}

float xsum, ysum, value;
#define EDGE_INTENSITY 150
#define EDGE(x, y) (img.getValue((x),(y)) > EDGE_INTENSITY)

// finds rightmost seed on the current line
static int 
findRightmostSeed(Coord *old, Coord *right, Image& img) 
{
  int x;
  for (x=int(old->x); !EDGE(x, int(old->y)) && (x < img.width()-1); x++) {
    // sanity, if x outside the image
    if (x >= img.width()-1) {
      return 0;
    }
  }
    
  right->y = old->y;
  right->x = float(x-1);
  return 1;
}

// find next righmost seed either above (direction = 1) or 
// below (direction = -1) the current seed line.
static int 
findNextSeed(Coord *seed, Coord *next, int direction, Image &img)
{
  // going to the left check for interior pixels in the 
  // proper direction
#if 0
  for (int x=int(seed->x); !EDGE(x, int(seed->y)); x--) {
#else
  // Added 6/16/1998 SMR
  // Handle single-pixel noise
  for (int x=int(seed->x); !EDGE(x, int(seed->y)) || (!EDGE(x+1, int(seed->y)) && !EDGE(x-1, int(seed->y))); x--) {
#endif
    // check if inside image
    if (x <= 1) {
      showMsg("findNextSeed: seed line goes out of the image");
      return -1;
    }
	
    // look above (below) current seed
#if 0
    if (!EDGE(x, int(seed->y) + direction)) {
#else
    // Added 6/16/1998 SMR
    // Handle single-pixel noise
    if (!EDGE(x, int(seed->y) + direction) && !EDGE(x-1, int(seed->y) + direction)) {
#endif
      // take the righmost seed on that line
      Coord tmp;
      tmp.x = float(x);
      tmp.y = seed->y+direction;
      if (!findRightmostSeed(&tmp, next, img)) {
	showMsg("findNextSeed: rightmost seed out of image");
	return -1;
      }
      return 1;
    }
  }
    
  // still here, no more seeds in this direction
  return 0;
}

// fill the line with the seed
// since we made sure seed is the next to the right edge 
// we need only go left
static int 
fillSeedLine(Coord *seed, Coord *min, Coord *max, Image& img,
	     Image& rgbimg)
{
  int x;
	
  // sanity checks
  if(EDGE(int(seed->x), int(seed->y))) {
    showMsg("fillSeedLine: seed is already on the border line");
    return 0;
  }	
  if(!EDGE(int(seed->x)+1, int(seed->y))) {
    showMsg("fillSeedLine: seed is not next to the right edge");
    return 0;
  }	
	
  // update corners if necessary
  min->y = MIN(seed->y, min->y);
  max->y = MAX(seed->y, max->y);
  max->x = MAX(seed->x, max->x);
	
  // go left
#if 0
  for (x=int(seed->x); !EDGE(x, int(seed->y)); x--) {
#else
  // Added 6/16/1998 SMR
  // Handle single-pixel noise
  for (x=int(seed->x); !EDGE(x, int(seed->y)) || (!EDGE(x+1, int(seed->y)) && !EDGE(x-1, int(seed->y))); x--) {
#endif
    // sanity, if x outside the image
    if (x <= 1) {
      showMsg("fillSeedLine: seed line goes out of the image");
      return 0;
    } 

    xsum += float(x);
    ysum += seed->y;
    value++;
    int pos = int(seed->y)*img.width()+x;
    rgbimg[3*pos]   = 0;
    rgbimg[3*pos+1] = 180;
    rgbimg[3*pos+2] = 0;
  }

  min->x = MIN(x+1, min->x);
  return 1;
}

	
static int 
findBoundingBox(Coord *seed, Image& img, Image& rgbimg)
{
  Coord min, max;
  int direction;
  Coord next;
  int status;
        
  // initialize corners with the seed
  min = max = *seed;

  // find rightmost seed on the seed line
  // need to do this just the first time to simplify the code.
  // from now on every new seed will be on the rightmost edge
  if (!findRightmostSeed(seed, seed, img)) {
    showMsg("findBoundingBox: righmost seed out of image");
    return 0;
  }
    
  next = *seed;
    
  // go up from next
  direction = +1;
  do {
    if (!fillSeedLine(&next, &min, &max, img, rgbimg)) {
      cerr << "findBoundingBox: "
	   << "error filling seed line, going up" << endl;
      return 0;
    }
  } while ((status = findNextSeed(&next, &next, direction, img)) == 1);
			
  if (status == -1) {
    cerr << "findBoundingBox: going up" << endl;
    return 0;
  }

  // go down from seed
  next = *seed;
	
  direction = -1;
  while ((status = findNextSeed(&next, &next, direction, img)) == 1) {
    if (!fillSeedLine(&next, &min, &max, img, rgbimg)) {
      cerr << "findBoundingBox: "
	   << "error filling seed line, going down" << endl;
      return 0;
    }
  }

  if (status == -1) {
    cerr << "findBoundingBox: going down" << endl;
    return 0;
  }

  return 1;
}


// using the initial approximation for center (old) 
// find all dark pixels beinting
// to the circle and average them to obtain better center
int 
findBetterCenter(Coord *old, Coord *better, Image& img, 
		 Image& rgbimg)
{

  xsum = ysum = value = 0.0f;

  if(!findBoundingBox(old, img, rgbimg)) {
    return 0;
  }
	
  if (value != 0.0) {
    better->x = xsum / value;
    better->y = ysum / value;
  } else {
    return 0;
  }
  return 1;
}	

const int BORDER = 3;
 
double 
dot(Coord a, Coord b)
{ 
  return a.x*b.x+a.y*b.y;
}

double 
angle(Coord a, Coord o, Coord b)
{
  Coord A,B;
  A.x = a.x - o.x; A.y = a.y - o.y;
  B.x = b.x - o.x; B.y = b.y - o.y;
  return acos(dot(A,B)/(hypot(A.x,A.y)*hypot(B.x,B.y)));
}

inline int
VGRAD(int width, int a, int b, Image& img)
{
  return (int(img[width*(a-3)+b])+
	  int(img[width*(a-2)+b])+
          int(img[width*(a-1)+b])-
	  int(img[width*(a)+b])  -
	  int(img[width*(a+1)+b])-
          int(img[width*(a+2)+b]))/3L;
}
inline int
HGRAD(int width, int a, int b, Image &img)
{
  return (int(img[width*a+b-3])+
	  int(img[width*a+b-2])+
	  int(img[width*a+b-1])-
	  int(img[width*a+b])  -
	  int(img[width*a+b+1])-
	  int(img[width*a+b+2]))/3L;
}

static int
closestDot(Coord *c, int *used, int size, Coord target)
{
  float dx, dy, dSqr, mindSqr = 1.e10f;
  int answer = -1;
  for (int i=0; i<size; i++) {
    if (!used[i]) {
      dx = target.x - c[i].x;
      dy = target.y - c[i].y;
      dSqr = dx*dx + dy*dy;
      if (dSqr < mindSqr) {
	mindSqr = dSqr;
	answer = i;
      }
    }
  }
  if (target.x > 1.0 && target.y > 1.0 && mindSqr > 2500.0) {
    // someone else but startpoint was >50 pixels off the target
    answer = -1;
  }
  if (answer != -1) used[answer] = 1;
  return answer;
}

// accumulate, but don't overflow
void
acc_uchar(unsigned char &acc, int vote)
{
  if (int(acc) + vote > 255) acc = 255;
  else                       acc += vote;
}

int
hough(unsigned char *hAcc,
      unsigned char *vAcc,
      int width,
      int height,
      Coord *coords,
      int rows,
      int cols,
      int R,
      int dR,
      Image& img)
{
  int i,j,k,l;
  // clear the accumulator
  for (i=0; i<width*height; i++)
    hAcc[i] = vAcc[i] = 0;
  //=================================================
  // voting
  //=================================================
  int gradient;
  const int gradThresh = 70;
  	
  for (i=BORDER; i<height-BORDER; i++) {
    for (j=BORDER; j<width-BORDER; j++) {
      // calculate horizontal gradient, which must be big...
      if ((gradient = HGRAD(width, i,j,img)) < gradThresh)
	continue;
      			
      // process only if next one doesn't have even bigger gradient
      if (HGRAD(width, i,j+1,img) > gradient)
	continue;
				
      // find the right edge
      k = BORDER;
      int done = 0;
      do {
	k++;
	if (k > (R+dR)*2 || j+k+BORDER >= width) 
	  break; // went too far
	if ((gradient = HGRAD(width, i,j+k,img)) < -gradThresh) {
	  // check whether the next one is even better
	  if (HGRAD(width, i,j+k+1,img) < gradient) k++;
	  done++;
	}
      } while (!done);
      // if not too big and not too small and within image...
      if (k <= (R+dR)*2 && k > R && j+k+BORDER < width) {
	// vote into the the hAcc
	int center = j+k/2;
	// mark boundaries in frame buffer
	//img[i*width+j] = 1;
	//img[i*width+j+k] = 1;

	for (l=i-R-dR; l<=i+R+dR; l++) {
	  if (l>0 && l<height) {
	    int loc = l*width+center;
	    if (k%2) { // k is odd
	      acc_uchar(hAcc[loc],   5);
	      acc_uchar(hAcc[loc-1], 3);
	      acc_uchar(hAcc[loc+1], 3);
	      acc_uchar(hAcc[loc-2], 1);
	      acc_uchar(hAcc[loc+2], 1);
	    } else {   // k is even
	      acc_uchar(hAcc[loc],   4);
	      acc_uchar(hAcc[loc-1], 4);
	      acc_uchar(hAcc[loc+1], 2);
	      acc_uchar(hAcc[loc-2], 2);
	    }
	  }
	}
	// skip what we have already covered
	j += k;
      }
    }
  }

  // check the vertical gradient
  for (j=BORDER; j<width-BORDER; j++) {
    for (i=BORDER; i<height-BORDER; i++) {
      if ((gradient = VGRAD(width, i,j,img)) < gradThresh) 
	continue;
      // process only if next one doesn't have even bigger gradient
      if (VGRAD(width, i+1,j,img) > gradient) 
	continue;
		  
      // find the bottom edge
      k = BORDER;
      int done = 0;
      do {
	k++;
	if (k > (R+dR)*2 || i+k+BORDER >= height) 
	  break; // went too far
	if ((gradient = VGRAD(width, i+k,j,img)) < -gradThresh) {
	  // check whether the next one is even better
	  if (VGRAD(width, i+k+1,j,img) < gradient) k++;
	  done++;
	}
      } while (!done);

      // if not too big and not too small and within image...
      if (k <= (R+dR)*2 && k > R && i+k+BORDER < height) {
	// vote into the the vAcc
	int center = i+k/2;
		    
	// mark boundaries in frame buffer
	//img[i*width+j] = 1;
	//img[(i+k)*width+j] = 1; 
		    
	for (l=j-R-dR; l<=j+R+dR; l++) {
	  if (l>0 && l<width) {
	    int loc = center*width+l;
	    if (k%2) { // k is odd
	      acc_uchar(vAcc[loc],   5);
	      acc_uchar(vAcc[loc-1*width], 3);
	      acc_uchar(vAcc[loc+1*width], 3);
	      acc_uchar(vAcc[loc-2*width], 1);
	      acc_uchar(vAcc[loc+2*width], 1);
	    } else {   // k is even
	      acc_uchar(vAcc[loc],         4);
	      acc_uchar(vAcc[loc-1*width], 4);
	      acc_uchar(vAcc[loc+1*width], 2);
	      acc_uchar(vAcc[loc-2*width], 2);
	    }
	  }
	}
		    	
	// skip what we have alread covered
	i += k;
      }
    }
  }     
	
  //=================================================
  // find local maxima
  //=================================================
  // choose minimum of hAcc and vAcc into hAcc
  for (i=0; i<width*height; i++)
    if (vAcc[i] < hAcc[i]) hAcc[i] = vAcc[i];

  // suppress all but local maxima, put them into vAcc
  // maxima have to be big enough                      
#if 0
  const int maximaDist = 15;
#else
  // SMR
  const int maximaDist = 5;
#endif
  //const int locMaxThreshold = 12;
  const int locMaxThreshold = 25;
  for (i=maximaDist; i<height-maximaDist; i++) {
    for (j=maximaDist; j<width-maximaDist; j++) {
      int loc = i*width+j;
      if (hAcc[loc]>locMaxThreshold){
	vAcc[loc] = hAcc[loc];
	for (k=-maximaDist; k<=maximaDist; k++) {
	  for (l=-maximaDist; l<=maximaDist; l++) {
	    int tmp = loc + k*width + l;
	    if (hAcc[loc] < hAcc[tmp])
	      vAcc[loc] = 0;
	  }
	}
      } else {
	vAcc[loc] = 0;
      }
    }
  }
  // find the accurate locations of the maxima
  // vAcc contains the maxima, hAcc the votes
  // vAcc is cleared in the process
  const int arrSize = 2000;
  Coord maxima[arrSize];
  int   mInd = 0;
  for (i=maximaDist; i<height-maximaDist; i++) {
    for (j=maximaDist; j<width-maximaDist; j++) {
      int loc = i*width+j;
      if (vAcc[loc]) {
	// this is a maximum, 
	// calculate a weighted sum for its location
	int sum = 0, sumk = 0, suml = 0;
	for (k=-maximaDist; k<=maximaDist; k++) {
	  for (l=-maximaDist; l<=maximaDist; l++) {
	    int tmp = loc + k*width + l;
	    vAcc[tmp] = 0;
	    sumk += k*hAcc[tmp];
	    suml += l*hAcc[tmp];
	    sum  += hAcc[tmp];
	  }
	}
	maxima[mInd].x = float(suml)/float(sum) + float(j);
	maxima[mInd].y = float(sumk)/float(sum) + float(i);
	//use for debugging
#if DEBUG
	SHOW(maxima[mInd]);
#endif
	if (++mInd >= arrSize) {
	  showMsg("Too many maxima");
	  fprintf(stderr, "mind = %d, arrSize = %d\n",mInd, arrSize);
	  return 0;
	}
      }
    }
  }
  if (mInd < rows*cols) {
    char errString[200];
    sprintf(errString,"Found only %ld maxima, %ld needed, hough()",
	    mInd, rows*cols);
    showMsg(errString);
    return 0;
  }                             
  //=================================================
  // find the point grid
  //=================================================
  // the 'used' array denotes whether a maximum is already chosen
  // to be a dot
  int used[arrSize];  
  for (i=0; i<mInd; i++) used[i] = 0;
  Coord target; target.x = 0.0f; target.y = 0.0f;
  // find the maximum that's closest to the upper left corner
  // find the 3 closest maxima to the start maximum
  float dx, dy;
  int done = 0;
  int startIndex;
  int closest[3];
  while (!done && mInd >= rows*cols) {
    startIndex = closestDot(maxima, used, mInd, target);
    float dSqrMin[3] = {1.e10f, 1.e10f, 1.e10f};
    float dSqr;
    for (i=0; i<mInd; i++) {
      if (!used[i]) {
	dx = maxima[i].x - maxima[startIndex].x;
	dy = maxima[i].y - maxima[startIndex].y;
	dSqr = dx*dx + dy*dy;
	if (dSqr < dSqrMin[2]) {
	  if (dSqr < dSqrMin[1]) {
	    dSqrMin[2] = dSqrMin[1];
	    closest[2] = closest[1];
	    if (dSqr < dSqrMin[0]) {
	      dSqrMin[1] = dSqrMin[0];
	      closest[1] = closest[0];
	      dSqrMin[0] = dSqr;
	      closest[0] = i;
	    } else {
	      dSqrMin[1] = dSqr;
	      closest[1] = i;
	    }
	  } else {
	    dSqrMin[2] = dSqr;
	    closest[2] = i;
	  }
	}
      }
    }
    // test whether these are good points
    // if not, find a new starting point
    // test the angle of the 2 closest points, must be close to 90 
    // test the distance of the third point from where it's expected to be         
    Coord o = maxima[startIndex];
    Coord a = maxima[closest[0]];
    Coord b = maxima[closest[1]];
#if 0
    Coord c = maxima[closest[2]];
    if (hypot(a.x+b.x-o.x-c.x, a.y+b.y-o.y-c.y) > 30.0 // the 3rd point's too far
	||
#else
    // SMR - handle not-really-square grids. Less robust, but more likely
    // to give a solution. Double check all your solutions!
    if (
#endif
	fabs(angle(a,o,b) - 1.571) > 0.7 // not a right angle
	) {
      // remove the bad starting point
      maxima[startIndex] = maxima[mInd-1];
      mInd--;
      used[closest[0]] = 0;
      used[closest[1]] = 0;
      used[closest[2]] = 0;
      used[startIndex] = 0;
    } else {
      done++;
    }
  }                                
  if (!done) {
	fprintf(stderr, "Couldn't find a place to start.\n");
	return 0;
  }
  // find the next ones in the same row and same column
  float mindx = 1.e10f, mindy = 1.e10f;
  int rowStart, colStart;
  for (i=0; i<3; i++) {
    dx = (float) fabs(maxima[startIndex].x - maxima[closest[i]].x);
    if (dx < mindx) {
      mindx = dx;
      colStart = closest[i];
    }
    dy = (float) fabs(maxima[startIndex].y - maxima[closest[i]].y);
    if (dy < mindy) {
      mindy = dy;
      rowStart = closest[i];
    }
  }
  // the outindex array denotes which maxima was accepted for a dot
  int *outindex = new int[rows*cols];
  outindex[0] = startIndex;
  outindex[1] = rowStart;
  // find the first row
  for (i=2; i<cols; i++) {
    target.x = 2*maxima[outindex[i-1]].x - maxima[outindex[i-2]].x;
    target.y = 2*maxima[outindex[i-1]].y - maxima[outindex[i-2]].y;
    outindex[i] = closestDot(maxima, used, mInd, target);
  }
  outindex[cols] = colStart;
  // find the first column
  for (i=2; i<rows; i++) {
    target.x = 2*maxima[outindex[(i-1)*cols]].x 
               - maxima[outindex[(i-2)*cols]].x;
    target.y = 2*maxima[outindex[(i-1)*cols]].y 
               - maxima[outindex[(i-2)*cols]].y;
    outindex[i*cols] = closestDot(maxima, used, mInd, target);
  }
  // find the rest
  for (i=1; i<rows; i++) {
    for (j=1; j<cols; j++) {
      target.x = maxima[outindex[i*cols+j-1]].x 
	       + maxima[outindex[(i-1)*cols+j]].x 
	       - maxima[outindex[(i-1)*cols+j-1]].x;
      target.y = maxima[outindex[i*cols+j-1]].y 
	       + maxima[outindex[(i-1)*cols+j]].y 
	       - maxima[outindex[(i-1)*cols+j-1]].y;
      outindex[i*cols+j] = closestDot(maxima, used, mInd, target);
    }
  }                
  int status = 1;
  for (i=0; i<rows*cols; i++) {
    if (outindex[i] == -1) {
      status = 0;
      break;
    }
    coords[i] = maxima[outindex[i]];
  }

  delete outindex;
  return status;
}


// find the circle centers in the calibration pattern
void 
findCenters(int rows, int cols, int R, int dR, 
	    Image& img, Image& rgbimg, vector<Coord>& pts)
{
  Image hAcc(img.width(), img.height(), 1);
  Image vAcc(img.width(), img.height(), 1);
  Coord *coord  = new Coord[rows*cols];
  Coord *better = new Coord[rows*cols];

  if (!hough(hAcc, vAcc, img.width(), img.height(), coord, rows, 
	     cols, R, dR,img))
    {
      hAcc.write("hacc.rgb");
      vAcc.write("vacc.rgb");
      cerr << "Failed to find hough points" << endl;
      for (int i=0; i<rows*cols; i++)
	{
	  int hough_pos = int(coord[i].y+.5)*img.width()+
	    int(coord[i].x+.5);
#if 0
	  SHOW(coord[i]);
#endif
	  if (hough_pos >= 0 && hough_pos <
	      img.width()*img.height())
	    {
	      for (int u=-1; u<=1; u++) 
		{
		  for (int v=-1; v<=1; v++)
		    {
		      int disp = u*img.width()-v;
		      // mark hough center with a red 3x3 square
		      rgbimg[3*(hough_pos+disp)]   = 180;
		      rgbimg[3*(hough_pos+disp)+1] = 0;
		    }
		}
	    }
	}
      
    } //ends if (!hough) block
  else
    {                
      for (int j=0; j<cols*rows; j++)
	{
	  int hough_pos = int(coord[j].y+.5)*img.width()+
	    int(coord[j].x+.5);
	  int mass_pos;
	
	  if(findBetterCenter(&coord[j], &better[j],
			      img, rgbimg))
	    {
	      if ((better[j].x > img.width()) || 
		  (better[j].y > img.height()))
		{
		  cerr <<"better ctr outside the bounds" << endl;
		}
	      else
		{
		  mass_pos = int(better[j].y+.5)*img.width()+
		    int(better[j].x+.5);
		  //write better[j] (type Coord) to a
		  //file or vector
		  //this is done below
		}
	    }
	  else
	    {
	      cerr << "Failed to find mass points" << endl;
	      mass_pos = hough_pos;
	    }
	  for (int u=-1; u<=1; u++) 
	    for (int v=-1; v<=1; v++)
	      {
		//this is kari's old drawing code that
		//seems to fail, with some model 15scans
		//i think my image.cc objects to the []
		//notation....b/c that same notation
		//seg faulted when writing to an image
		//in houghmain.cc
		//mark hough center with a red 3x3 square

		/*

		int cx, cy;
		cx = (int)coord[j].x+0.5;
		cy = (int)coord[j].y+0.5;
		if(rgbimg.inside(cx+u,cy+v))
		   {

		     rgbimg.setValue(cx+u,cy+v,180,0,
				     rgbimg.getB(cx+u,cy+v),0);
		     //mark mass center with a blue 3x3 square
		     rgbimg.setValue(cx+u,cy+v,
				     rgbimg.getR(cx+u,cy+v),
				     0,180,0);
				     }

		*/
		
		int disp = u*img.width()-v;
		// mark hough center with a red 3x3 square
		rgbimg[3*(hough_pos+disp)]   = 180;
		rgbimg[3*(hough_pos+disp)+1] = 0;
		// mark mass center with a blue 3x3 square
		rgbimg[3*(mass_pos+disp)+1] = 0;
		rgbimg[3*(mass_pos+disp)+2] = 180;
		
	      }
#if 1
	  	// write the position to the points
		// vector - use mass position
		pts.push_back(better[j]);
#else
		pts.push_back(coord[j]);
#endif

		
	}
    }
  
  delete[] coord;
  delete[] better;
}


