#!/usr/common/bin/wish8.0
#############################################################################
# Visual Tcl v1.20 Project
#

#################################
# GLOBAL VARIABLES
#
global gInputImages; 
global gNumImages; 
global gR; 
global gdR; 
global numCols; 
global numRows; 
global spacing; 
global threshold; 
global widget; 
    set widget(canvas) {.top17.can18}
    set widget(rev,.top17.can18) {canvas}

#################################
# USER DEFINED PROCEDURES
#
proc init {argc argv} {
set prompt "tcl> "
global tcl_interactive
puts $tcl_interactive
set tcl_interactive 1


global env
source "$env(TCLCALIB_DIR)/tcl_util.tcl"
}

init $argc $argv


proc {Scroll_Set} {scrollbar geoCmd offset size} {
if {$offset != 0.0 || $size != 1.0} {
	eval $geoCmd;
	$scrollbar set $offset $size
    } else {
	set manager [lindex $geoCmd 0]
	$manager forget $scrollbar
    }
}

proc {Scrolled_Listbox} {f args} {
frame $f
    listbox $f.list  -yscrollcommand [list Scroll_Set $f.yscroll  [list grid $f.yscroll -row 0 -column 1 -sticky ns]]
    eval {$f.list configure} $args
    scrollbar $f.yscroll -orient vertical  -command [list $f.list yview]
    grid $f.list $f.yscroll -sticky news
    grid columnconfigure $f 0 -weight 1
    return $f.list
}

proc {alias} {newname oldname} {
eval "proc $newname args \{eval \"$oldname \$args\"\}"
}

proc {beep} {} {
# TODO: silent mode would bail here
    puts -nonewline "\07"
}

proc {centerWindow} {window} {
wm withdraw $window
    update idletasks

    set w [winfo reqwidth $window]
    set h [winfo reqheight $window]

    set x [expr ([winfo screenwidth  $window] - $w) / 2]
    set y [expr ([winfo screenheight $window] - $h) / 2]

    wm geometry $window "+$x+$y"
    wm deiconify $window
}

proc {createNamedCascade} {parent name caption {opts {}}} {
set child [menu $parent.$name  -tearoffcommand "setMenuTitle \"$caption\""]
    if {$opts == ""} {
	$parent add cascade -label $caption -menu $child
    } else {
	eval "$parent add cascade -label $caption -menu $child $opts"
    }
    return $child
}

proc {fixButtonColors} {args} {
# relevant entries are activebackground, background, highlightbackground
    # and activeforeground, foreground, highlightcolor, highlightthickness
    # and selectcolor

    # unselected: background, mouse makes it activebackground
    # selected: selectcolor, mouse makes it activebackground

    foreach widget $args {
	bind $widget <Enter>    { %W config -activebackground [getBG %W]}
	bind $widget <Button-1> { %W config -activebackground [getBG %W  yes]}
    }
}

proc {flash} {var value} {
upvar $var flashee
    set old [set flashee]
    set flashee $value
    set flashee $old
}

proc {getBG} {widget {force no}} {
set on [lindex [$widget config -value] 4]
    if {[globalset [lindex [$widget config -variable] 4]] == "$on"
	|| $force == "yes"} {
	set bg -selectcolor
	set ofs 4
    } else {
	set bg -activebackground
	set ofs 3
    }

    set b [lindex [$widget config $bg] $ofs]
    return $b
}

proc {getListboxSelection} {listbox} {
set sel [$listbox cursel]
    if {[string compare $sel ""] == 0} {
	return ""
    } else {
	return [$listbox get $sel]
    }
}

proc {getglobalname} {var} {
set av $var
    set arr [string first "(" $av]
    if {$arr == -1} {
	return $av
    }

    return [string range $av 0 [expr $arr - 1]]
}

proc {globalexists} {var} {
global $var
    return [info exists $var]
}

proc {globalinit} {var {val {}}} {
global $var
    uplevel "global $var"

    if {![info exists $var]} {
	set $var $val
    }
}

proc {globalset} {var {val ~}} {
global [getglobalname $var]

    if {$val != "~"} {
	set $var $val
    } else {
	set $var
    }
}

proc {globaltrace} {var access proc} {
global $var
    trace variable $var $access $proc
}

proc {ldelete} {list first {last {}}} {
if {$last == ""} {set last $first}

    set front [lrange $list 0 [expr $first - 1]]
    set tail [lrange $list [expr $last + 1] end]

    return [concat $front $tail]
}

proc {makeproc} {name args def} {
proc $name $args $def
}

proc {menuInvokeByName} {menu name} {
# make non-ambiguous string index out of name; otherwise, if
    # numeric, it'll be taken as a count-based index

    set v1 [string index $name 0]
    set vr [string range $name 1 end]
    set name \[$v1\]$vr

    set ind [$menu index $name]
    $menu invoke $ind
    return $ind
}

proc {packchildren} {parent args} {
foreach child [winfo children $parent] {
	eval pack $child $args
    }
}

proc {prebind} {widget binding script} {
set oldscript [bind $widget $binding]
    bind $widget $binding $script
    bind $widget $binding +$oldscript
}

proc {repeat} {n script} {
for {set i 0} {$i < $n} {incr i} {eval $script}
}

proc {resetBGcolor} {widget} {
# resets a widget's background color to that of its parent
    set bg [lindex [[winfo parent $widget] config -bg] 4]
    $widget config -bg $bg
}

proc {swap} {var1 var2} {
upvar $var1 v1
    upvar $var2 v2

    set swap $v2
    set v2 $v1
    set v1 $swap
}

proc {verboseOptionMenu} {w caption varName args} {
upvar #0 $varName var
    
    menubutton $w -text $caption -indicatoron 1 -menu $w.menu  -relief raised -bd 2 -highlightthickness 2 -anchor c  -direction flush -padx 0 -pady 0
    menu $w.menu -tearoff 0
    foreach i $args {
	set val  [lindex $i 0]
	if {[llength $i] > 1} {
	    set desc [lindex $i 1]
	} else {
	    set desc $val
	}
        $w.menu add radiobutton -label $desc  -variable $varName -value $val
    }
    return $w.menu
}

proc {main} {argc argv} {
global gInputImages
global gNumImages
set gInputImages $argv
set gNumImages $argc
eval ".top17.lis20 insert end $argv"



globalset numRows 8
globalset numCols 8
globalset gR 15
globalset gdR 10
globalset threshold 100
globalset spacing 65
}

proc {Window} {args} {
global vTcl
    set cmd [lindex $args 0]
    set name [lindex $args 1]
    set newname [lindex $args 2]
    set rest [lrange $args 3 end]
    if {$name == "" || $cmd == ""} {return}
    if {$newname == ""} {
        set newname $name
    }
    set exists [winfo exists $newname]
    switch $cmd {
        show {
            if {$exists == "1" && $name != "."} {wm deiconify $name; return}
            if {[info procs vTclWindow(pre)$name] != ""} {
                eval "vTclWindow(pre)$name $newname $rest"
            }
            if {[info procs vTclWindow$name] != ""} {
                eval "vTclWindow$name $newname $rest"
            }
            if {[info procs vTclWindow(post)$name] != ""} {
                eval "vTclWindow(post)$name $newname $rest"
            }
        }
        hide    { if $exists {wm withdraw $newname; return} }
        iconify { if $exists {wm iconify $newname; return} }
        destroy { if $exists {destroy $newname; return} }
    }
}

#################################
# VTCL GENERATED GUI PROCEDURES
#

proc vTclWindow. {base} {
    if {$base == ""} {
        set base .
    }
    ###################
    # CREATING WIDGETS
    ###################
    wm focusmodel $base passive
    wm geometry $base 1x1+0+0
    wm maxsize $base 1585 1170
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm withdraw $base
    wm title $base "vt.tcl"
    ###################
    # SETTING GEOMETRY
    ###################
}

proc vTclWindow.top17 {base} {
    if {$base == ""} {
        set base .top17
    }
    if {[winfo exists $base]} {
        wm deiconify $base; return
    }
    ###################
    # CREATING WIDGETS
    ###################
    toplevel $base -class Toplevel \
        -menu .top17.m58 
    wm focusmodel $base passive
    wm geometry $base 671x788+197+194
    wm maxsize $base 1585 1170
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "calib"
    canvas $base.can18 \
        -borderwidth 2 -height 480 -relief ridge -width 640 
    listbox $base.lis20 \
        -selectmode extended 
    button $base.but21 \
        \
        -command #\ First\ the\ main\ command\nglobal\ env\nset\ str\ \"path\ ('\$env(TCLCALIB_DIR)/../heikkila/',path)\;\ \\\n\ \ feval('match2fin','camera','.'\"\n\n\ \ \n#\ Have\ we\ actually\ got\ any\ matchfiles\nset\ matchfilescount\ 0\n\ \ \n#\ add\ the\ image\ file\ parameters\ on\ at\ a\ time\nset\ sel\ \[.top17.lis20\ curselection\]\nforeach\ file\ \$sel\ \{\n\ \ \ \ set\ filename\ \[.top17.lis20\ get\ \$file\]\n\ \ \ \ set\ basefile\ \[file\ rootname\ \[file\ tail\ \$filename\]\]\n\ \ \ \ set\ matchfile\ \"\$basefile.match\"\n\ \ \ \ #puts\ \"\$basefile\ \$matchfile\"\n\ \ \ \ #if\ the\ match\ file\ exists\n\ \ \ \ if\ \[file\ exists\ \$matchfile\]\ \{\n\ \ \ \ \ \ \ \ set\ str\ \"\$str,'\$basefile'\"\n\ \ \ \ \ \ \ \ incr\ matchfilescount\n\ \ \ \ \}\n\n\}\ \n\n#\ add\ the\ trailing\ stuff\nset\ str\ \"\$str\ )\;\"\n\n#\ send\ the\ string\ to\ matlab\nif\ \{\$matchfilescount\ !=\ 0\}\ \{\n\ \ \ \ puts\ \$str\n\ \ \ \ exec\ echo\ \$str\ |\ matlab\ \ &\n\}\ else\ \{\n\ \ \ \ puts\ \"No\ matchfiles\ selected\ to\ calibrate\ from.\"\n\} \
        -padx 14 -pady 5 -text Callibrate 
    scale $base.sca27 \
        -orient horiz -variable spacing 
    scale $base.sca28 \
        -orient horiz -to 200.0 -variable threshold 
    scale $base.sca29 \
        -orient horiz -variable gdR 
    scale $base.sca30 \
        -orient horiz -to 30.0 -variable gR 
    scale $base.sca31 \
        -orient horiz -to 30.0 -variable numCols 
    label $base.lab32 \
        -borderwidth 1 -text {Spacing (mm)} 
    label $base.lab33 \
        -borderwidth 1 -text Threshold 
    label $base.lab34 \
        -borderwidth 1 -text {dR (pixels)} 
    label $base.lab35 \
        -borderwidth 1 -text {R (pixels)} 
    label $base.lab36 \
        -borderwidth 1 -text {Num Cols} 
    button $base.but37 \
        \
        -command set\ sellist\ \[.top17.lis20\ curselection\]\nif\ \{\$sellist\ !=\ \"\"\}\ \{\n\ \ \ \ #\ Get\ the\ filename\n\ \ \ \ set\ filename\ \[.top17.lis20\ get\ \$sellist\]\n\ \ \ \ puts\ \"\$filename\ \$numRows\ \$numCols\"\n\ \ \ \ global\ env\n\ \ \ \ set\ str\ \"\$env(TCLCALIB_DIR)/../hough/hough\ \$numRows\ \\\n\ \ \ \ \ \ \$numCols\ \$gR\ \$gdR\ \$threshold\ \$filename\"\n\ \ \ \ puts\ \$str\n\ \ \ \ set\ filetail\ \[file\ tail\ \$filename\]\n\ \ \ \ set\ outfile\ \"houghout_\$filetail\"\n\ \ \ \ #\ Remove\ the\ old\ output\ file\n\ \ \ \ puts\ \"rm\ \$outfile\"\n\ \ \ \ eval\ \"exec\ rm\ -f\ \$outfile\"\n\ \ \ \ #\ Run\ the\ circle\ searcher\n\ \ \ \ set\ errval\ \[catch\ \{eval\ \"exec\ \$str\"\}\]\n\ \ \ \ \n\ \ \ \ #\ Show\ the\ outfile\ or\ infile\n\ \ \ \ if\ \[file\ exists\ \$outfile\]\ \{\n\ \ \ \ \ \ \ \ set\ showimage\ \$outfile\n\ \ \ \ \}\ else\ \{\n\ \ \ \ \ \ \ \ set\ showimage\ \$filename\n\ \ \ \ \}\ \ \ \n\ \ \ \ \n\ \ \ \ \n\ \ \ \ #\ convert\ the\ image\ to\ viewable\ format\n\ \ \ \ exec\ convert\ \$showimage\ calib_tmp_image.ppm\n\ \ \ \ \n\ \ \ \ #\ show\ the\ image\ \ \ \ \ \n\ \ \ \ image\ create\ photo\ my_image\ -file\ calib_tmp_image.ppm\n\ \ \ \ .top17.can18\ create\ image\ 0\ 0\ -anchor\ nw\ -image\ my_image\n\ \n\ \ \ \ #\ if\ no\ err\ then\ make\ a\ 5tuple\ file\n\ \ \ \ set\ twoDpoints\ \"_points\"\n\ \ \ \ set\ fileroot\ \[file\ rootname\ \$filetail\]\n\ \ \ \ set\ twoDpoints\ \"\$filetail\$twoDpoints\"\n\ \ \ \ set\ tupleFile\ \"\$fileroot.match\"\n\ \ \ \ exec\ rm\ -f\ \$tupleFile\n\ \ \ \ if\ \{\$errval\ ==\ \"0\"\}\ \{\n\ \ \ \ \ \ \n\ \ \ \ \ \ \ \ set\ str\ \"/u/xcchen/proj/mtrack/ModelTrack/calib/util/match2ideal\ \\\n\ \ \ \ \ \ \ \ \ \ \ \ \$twoDpoints\ \$spacing\ \$numCols\ >\ \$tupleFile\"\n\ \ \ \ \ \ \ \ puts\ \$str\n\ \ \ \ \ \ \ \ eval\ \"exec\ \$str\"\n\ \ \ \ \}\n\} \
        -padx 14 -pady 5 -text {Find Dots} 
    scale $base.sca44 \
        -orient horiz -to 30.0 -variable numRows 
    label $base.lab57 \
        -borderwidth 1 -text {Num Rows} 
    menu $base.m58 \
        -cursor {} 
    button $base.but17 \
        -command exit -padx 11 -pady 4 -text quit 
    ###################
    # SETTING GEOMETRY
    ###################
    place $base.can18 \
        -x 10 -y 15 -anchor nw -bordermode ignore 
    place $base.lis20 \
        -x 190 -y 510 -width 273 -height 261 -anchor nw -bordermode ignore 
    place $base.but21 \
        -x 520 -y 585 -anchor nw -bordermode ignore 
    place $base.sca27 \
        -x 75 -y 730 -anchor nw -bordermode ignore 
    place $base.sca28 \
        -x 75 -y 685 -anchor nw -bordermode ignore 
    place $base.sca29 \
        -x 75 -y 640 -anchor nw -bordermode ignore 
    place $base.sca30 \
        -x 75 -y 595 -anchor nw -bordermode ignore 
    place $base.sca31 \
        -x 75 -y 550 -width 106 -height 43 -anchor nw -bordermode ignore 
    place $base.lab32 \
        -x 10 -y 750 -anchor nw -bordermode ignore 
    place $base.lab33 \
        -x 10 -y 705 -anchor nw -bordermode ignore 
    place $base.lab34 \
        -x 10 -y 660 -anchor nw -bordermode ignore 
    place $base.lab35 \
        -x 10 -y 615 -anchor nw -bordermode ignore 
    place $base.lab36 \
        -x 10 -y 570 -anchor nw -bordermode ignore 
    place $base.but37 \
        -x 520 -y 525 -anchor nw -bordermode ignore 
    place $base.sca44 \
        -x 75 -y 510 -width 106 -height 48 -anchor nw -bordermode ignore 
    place $base.lab57 \
        -x 10 -y 525 -anchor nw -bordermode ignore 
    place $base.but17 \
        -x 530 -y 640 -anchor nw -bordermode ignore 
}

Window show .
Window show .top17

main $argc $argv
